"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CertificateService = void 0;
const BootstrapUtils_1 = require("./BootstrapUtils");
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const path_1 = require("path");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class CertificateService {
    constructor(root, params) {
        this.root = root;
        this.params = params;
    }
    getKey(stdout, from, to) {
        const key = stdout
            .substring(stdout.indexOf(from) + from.length, stdout.indexOf(to))
            .trim()
            .split(':')
            .map((m) => m.trim())
            .join('');
        if (!key || key.length !== 64) {
            throw Error(`SSL Certificate key '${from}' cannot be loaded from the openssl script. Output: \n${stdout}`);
        }
        return key;
    }
    async run(name) {
        // Currently the SSL certifcates are created via a docker image. Migrate this to a native forge!
        // https://www.npmjs.com/package/node-forge
        const presetData = BootstrapUtils_1.BootstrapUtils.loadPresetData(this.root, this.params.preset, this.params.assembly, this.params.customPreset);
        const symbolServerToolsImage = presetData.symbolServerToolsImage;
        const copyFrom = `${this.root}/config/cert`;
        const target = `${this.params.target}/config/${name}/resources/cert`;
        await BootstrapUtils_1.BootstrapUtils.mkdir(target);
        const generatedContext = { name };
        const templateContext = Object.assign(Object.assign({}, presetData), generatedContext);
        await BootstrapUtils_1.BootstrapUtils.generateConfiguration(templateContext, copyFrom, target);
        const command = this.createCertCommands('/data');
        await BootstrapUtils_1.BootstrapUtils.writeTextFile(target + '/createCertificate.sh', command);
        const cmd = ['bash', '-c', 'cd /data && bash createCertificate.sh'];
        const binds = [`${path_1.resolve(target)}:/data:rw`];
        const userId = await BootstrapUtils_1.BootstrapUtils.getDockerUserGroup();
        const stdout = await BootstrapUtils_1.BootstrapUtils.runImageUsingExec(symbolServerToolsImage, userId, cmd, binds);
        const privateKey = this.getKey(stdout, 'priv:', 'pub:');
        const publicKey = this.getKey(stdout, 'pub:', 'Certificate:');
        logger.info(`Certificate for node ${name} created`);
        return { privateKey, publicKey };
    }
    createCertCommands(target) {
        return `
cd ${target}
set -e
mkdir new_certs && chmod 700 new_certs
touch index.txt.attr
touch index.txt

# create CA key
openssl genpkey -out ca.key.pem -outform PEM -algorithm ed25519
openssl pkey -inform pem -in ca.key.pem -text -noout
openssl pkey -in ca.key.pem -pubout -out ca.pubkey.pem

# create CA cert and self-sign it
openssl req -config ca.cnf -keyform PEM -key ca.key.pem -new -x509 -days 7300 -out ca.cert.pem
openssl x509 -in ca.cert.pem  -text -noout

# create node key
openssl genpkey -out node.key.pem -outform PEM -algorithm ed25519
openssl pkey -inform pem -in node.key.pem -text -noout

# create request
openssl req -config node.cnf -key node.key.pem -new -out node.csr.pem
openssl req  -text -noout -verify -in node.csr.pem

### below is done after files are written
# CA side
# create serial
openssl rand -hex 19 > ./serial.dat

# sign cert for 375 days
openssl ca -batch -config ca.cnf -days 375 -notext -in node.csr.pem -out node.crt.pem
openssl verify -CAfile ca.cert.pem node.crt.pem

# finally create full crt
cat node.crt.pem ca.cert.pem > node.full.crt.pem
`;
    }
}
exports.CertificateService = CertificateService;
