"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.RunService = void 0;
const LoggerFactory_1 = require("../logger/LoggerFactory");
const LogType_1 = require("../logger/LogType");
const child_process_1 = require("child_process");
const path_1 = require("path");
const symbol_sdk_1 = require("symbol-sdk");
const symbol_openapi_typescript_fetch_client_1 = require("symbol-openapi-typescript-fetch-client");
const BootstrapUtils_1 = require("./BootstrapUtils");
const fs_1 = require("fs");
const _ = require("lodash");
const logger = LoggerFactory_1.default.getLogger(LogType_1.LogType.System);
class RunService {
    constructor(params) {
        this.params = params;
    }
    async run() {
        const basicArgs = ['up', '--remove-orphans'];
        if (this.params.daemon) {
            basicArgs.push('--detach');
        }
        if (this.params.build) {
            basicArgs.push('--build');
        }
        await this.basicRun(basicArgs, false);
        if (this.params.daemon) {
            await this.pollServiceUntilIsUp(this.params.timeout || RunService.defaultParams.timeout || 0);
        }
    }
    async pollServiceUntilIsUp(totalPollingTime, pollIntervalMs = 10000) {
        const url = 'http://localhost:3000';
        const repositoryFactory = new symbol_sdk_1.RepositoryFactoryHttp(url);
        const nodeRepository = repositoryFactory.createNodeRepository();
        const pollFunction = async () => {
            logger.info(`Testing ${url}/node/health`);
            try {
                const healthStatus = await nodeRepository.getNodeHealth().toPromise();
                if (healthStatus.apiNode === symbol_openapi_typescript_fetch_client_1.NodeStatusEnum.Down) {
                    logger.warn(`Network is NOT up and running YET: Api Node is still Down!`);
                    return false;
                }
                if (healthStatus.db === symbol_openapi_typescript_fetch_client_1.NodeStatusEnum.Down) {
                    logger.warn(`Network is NOT up and running YET: DB is still Down!`);
                    return false;
                }
                logger.info(`Network IS up and running...`);
                return true;
            }
            catch (e) {
                logger.warn(`Network is NOT up and running YET: ${e.message}`);
                return false;
            }
        };
        const started = await BootstrapUtils_1.BootstrapUtils.poll(pollFunction, totalPollingTime, pollIntervalMs);
        if (!started) {
            throw new Error(`Network did NOT start!!!`);
        }
    }
    async stop() {
        await this.basicRun(['down'], true);
    }
    async basicRun(extraArgs, ignoreIfNotFound) {
        const dockerFile = path_1.join(this.params.target, `docker`, `docker-compose.yml`);
        const dockerComposeArgs = ['-f', dockerFile];
        const args = [...dockerComposeArgs, ...extraArgs];
        if (!fs_1.existsSync(dockerFile)) {
            if (ignoreIfNotFound) {
                logger.info(`Docker compose ${dockerFile} does not exist, ignoring: docker-compose ${args.join(' ')}`);
                return;
            }
            else {
                throw new Error(`Docker compose ${dockerFile} does not exist. Cannot run: docker-compose ${args.join(' ')}`);
            }
        }
        //Creating folders to avoid being created using sudo. Is there a better way?
        const dockerCompose = await BootstrapUtils_1.BootstrapUtils.loadYaml(dockerFile);
        const volumenList = _.flatMap(Object.values(dockerCompose === null || dockerCompose === void 0 ? void 0 : dockerCompose.services), (s) => { var _a; return ((_a = s.volumes) === null || _a === void 0 ? void 0 : _a.map((v) => v.split(':')[0])) || []; }) || [];
        await Promise.all(volumenList.map(async (v) => {
            await BootstrapUtils_1.BootstrapUtils.mkdir(path_1.join(this.params.target, `docker`, v));
        }));
        const cmd = child_process_1.spawn('docker-compose', args);
        const log = (data) => {
            console.log(`${data}`.trim());
        };
        cmd.stdout.on('data', (data) => {
            log(data);
        });
        cmd.stderr.on('data', (data) => {
            log(`${data}`.trim());
        });
        cmd.on('error', (error) => {
            log(`error: ${error.message}`.trim());
        });
        cmd.on('exit', (code, signal) => {
            log(`child process exited with code ${code} and signal ${signal}`);
        });
        cmd.on('close', (code) => {
            log(`child process exited with code ${code}`);
        });
        process.on('SIGINT', function () {
            log('Received SIGINT signal');
        });
        logger.info(`running: docker-compose ${args.join(' ')}`);
    }
}
exports.RunService = RunService;
RunService.defaultParams = { target: 'target', timeout: 60000 };
